
/* daemon.c 
 * corrections by chrissie 11/2019
 * http://www.microhowto.info/howto/cause_a_process_to_become_a_daemon_in_c.html#idp26576
 */
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <syslog.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <signal.h>
#include <stdlib.h>

static void start_daemon () {
       // Fork, allowing the parent process to terminate.
    pid_t pid = fork();
    if (pid == -1) {
        fprintf(stderr, "failed to fork while daemonising (errno=%d)",errno);
        exit(errno);
    } else if (pid != 0) {
        exit(0);
    }

    // Start a new session for the daemon.
    if (setsid()==-1) {
        fprintf(stderr, "failed to become a session leader while daemonising(errno=%d)",errno);
        exit(errno);
    }

    // Fork again, allowing the parent process to terminate.
    signal(SIGHUP,SIG_IGN);
    pid=fork();
    if (pid == -1) {
        fprintf(stderr, "failed to fork while daemonising (errno=%d)",errno);
        exit(errno);
    } else if (pid != 0) {
        exit(0);
    }

    // Set the current working directory to the root directory.
    if (chdir("/") == -1) {
        fprintf(stderr, "failed to change working directory while daemonising (errno=%d)",errno);
        exit(errno);
    }

    // Set the user file creation mask to zero.
    umask(0);

    // Close then reopen standard file descriptors.
    close(STDIN_FILENO);
    close(STDOUT_FILENO);
    close(STDERR_FILENO);
    if (open("/dev/null",O_RDONLY) == -1) {
        fprintf(stderr, "failed to reopen stdin while daemonising (errno=%d)",errno);
        exit (errno);
    }
    if (open("/dev/null",O_WRONLY) == -1) {
        fprintf(stderr, "failed to reopen stdout while daemonising (errno=%d)",errno);
        exit (errno);
    }
    if (open("/dev/null",O_RDWR) == -1) {
        fprintf(stderr, "failed to reopen stderr while daemonising (errno=%d)",errno);
        exit(errno);
    }
}

int main (int argc, char **argv) {
   int time = 15;
   start_daemon ();
   while (1) {
      /* Enlosschleifen: Hier sollte nun der Code für den
       * Daemon stehen, was immer er auch tun soll.
       * Bei Fehlermeldungen beispielsweise:
       * if(dies_ist_passiert)
       *    syslog(LOG_WARNING, "dies_ist_Passiert");
       * else if(das_ist_Passiert)
       *    syslog(LOG_INFO, "das_ist_Passiert");
       */
      syslog( LOG_NOTICE, "Daemon gestartet ...\n");
      sleep(time);
      syslog( LOG_NOTICE,
              "Daemon lief %d Sekunden\n", time );
      break;    /* endlosschleife verlassen, daemon beenden */
   }
   syslog( LOG_NOTICE, "Daemon hat sich beendet\n");
   closelog();
   return EXIT_SUCCESS;
}
